@@ -28,8 +28,8 @@ import operator
 import functools
 import inspect
 
-saved_dispatch = pickle.Pickler.dispatch.copy()
-saved_dispatch_table = pickle.dispatch_table.copy()
+#saved_dispatch = pickle.Pickler.dispatch.copy()
+#saved_dispatch_table = pickle.dispatch_table.copy()
 
 import types
 try:
@@ -972,24 +972,83 @@ class SPickleTools(object):
                 importModules.append(opcodes[1])
         return importModules        
           
-    def remotemethod(self, rpycconnection, method):
-        """Create a remote method.
+    CREATE_IMMEDIATELY = 'immedately'
+    """
+    Constant to be given to the `create_only_once` argument of method :meth:`remotemethod`. 
+    Create the function on the remote side during the invocation of :meth:`remotemethod`.
+    """
+
+    CREATE_LAZY = True
+    """
+    Constant to be given to the `create_only_once` argument of method :meth:`remotemethod`. 
+    Create the function on the remote side on the first invocation of the function returned by :meth:`remotemethod`.
+    The actual value is `True`.
+    """
+
+    CREATE_EVERYTIME = False
+    """
+    Constant to be given to the `create_only_once` argument of method :meth:`remotemethod`. 
+    Create the function on the remote side on every invocation of the function returned by :meth:`remotemethod`.
+    The actual value is `False`.
+    """
+
+    def remotemethod(self, rpycconnection, method=None, create_only_once=None):
+        """Create a remote function.
         
         This method takes an active RPyC connection and 
-        a locally defined method (or function) and returns 
+        a locally defined function (or method) and returns 
         a proxy for an equivalent function on the remote side.
         If you invoke the proxy, it will create a pickle containing the 
-        method and its parameter, transfer this pickle to the remote side, 
-        unpickle it and invoke the method. It then pickles the result 
-        and transfers the result back to the local side.
+        function, transfer this pickle to the remote side, 
+        unpickle it and invoke the function. It then pickles the result 
+        and transfers the result back to the local side. It will not pickle the 
+        function arguments. If you need to transfer the function arguments by 
+        value, use :func:`functools.partial` to apply them to your 
+        function prior to the call of remotemethod.
         
-        :param rpycconnection: an active RPyC connection
+        :param rpycconnection: an active RPyC connection. If set to `None`, execute
+                               method localy.
         :type rpycconnection: :class:`rpyc.core.protocol.Connection`
-        :param object method: a callable object
-        :return: the proxy for method
+        :param object method: a callable object. If you do not give this argument, 
+            you can use remotemethod as a decorator.
+        :param create_only_once: controlls the creation of the function on the 
+            remote side. If you want to create the function during the execution 
+            of :meth:`remotemethod`, pass :attr:`CREATE_IMMEDIATELY`. Otherwise,
+            if you want to create the remote function on its first invokation,
+            set create_only_once to a value that is `True` in a boolean context.
+            Otherwise, if you set create_only_once evaluates to `False`, 
+            the local proxy creates the create the remote 
+            function on every invocation.
+
+        :return: the proxy for the remote function
+        
+        .. note::
+           If you use `remotemethod` as a decorator, do not apply it on regular
+           methods of a class. It does not work in the desired way, because decorators 
+           work on the underlying function object, not on the method object. Therefore 
+           you will end up with a remote function, that recives a RPyC proxy for `self`.
+           
         """
+        if method is None:
+            return functools.partial(self.remotemethod, rpycconnection, create_only_once=create_only_once)
+
+        if create_only_once == self.CREATE_IMMEDIATELY:
+            rmethod_list = (self._build_remotemethod(rpycconnection, method),)
+        else:
+            rmethod_list = [bool(create_only_once)]
+
         def wrapper(*args, **keywords):
-            rmethod = self._build_remotemethod(rpycconnection, method)
+            if rpycconnection is None:
+                # the local case.
+                return method(*args, **keywords)
+
+            if callable(rmethod_list[0]):
+                rmethod = rmethod_list[0]
+            else:
+                rmethod = self._build_remotemethod(rpycconnection, method)
+                if rmethod_list[0] is True:
+                    rmethod_list[0] = rmethod
+
             r0, r1 = rmethod(*args, **keywords)
             if r0 is None:
                 return r1
@@ -998,14 +1057,21 @@ class SPickleTools(object):
             return self.loads_with_external_ids(r0, idmap)
         functools.update_wrapper(wrapper, method)
         return wrapper
-
                 
     def _build_remotemethod(self, rpycconnection, method):
         """return a remote method"""
         idmap = {}
         pickle = self.dumps_with_external_ids(method, idmap, matchResources=True)
-        rcls = rpycconnection.root.getmodule(self.__class__.__module__)
-        rcls = getattr(rcls, self.__class__.__name__)
+        try:
+            rcls = rpycconnection.root.getmodule(self.__class__.__module__)
+            rcls = getattr(rcls, self.__class__.__name__)
+        except ImportError, e:
+            LOGGER().debug("Remote side lacks the module, going to pickle it")
+            import sPickle._sPickle
+            pt = SPickleTools(serializeableModules=[sPickle, sPickle._sPickle])
+            # no compression to be compatible with the plain unpickler
+            rcls_pickled = pt.dumps((sPickle, self.__class__), doCompress=False)
+            rcls = rpycconnection.root.getmodule("cPickle").loads(rcls_pickled)[1]
         remotemethod = rcls()._build_remotemethod_remote(pickle, idmap)
         return remotemethod
     
@@ -1077,7 +1143,7 @@ class SPickleTools(object):
 class StacklessTaskletReturnValueException(BaseException):
     """This exception can be used to return a value from a Stackless Python tasklet.
     
-    Usually a tasklet has no way to return a value at the end of its live. But it can 
+    Usually a tasklet has no documented way to return a value at the end of its live. But it can 
     raise an exception. You can use this exception to encapsulate a return value.
     Because this exception is not a real error and you usually do not want 
     to catch this exception in normal "error" handling, it 
